package com.flow;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.view.WindowManager;

public class DeviceInfo {

    private final int dpi;

    private final double height;

    private final int heightPixels;

    private final String manufacturer;

    private final String model;

    private final String osName = "ANDROID";

    private final String osVersion;

    private final String appVersionName;

    private final String deviceName;

    private final double width;

    private final int widthPixels;

    private final transient Context context;

    DeviceInfo(Context context) {
        this.context = context;
        appVersionName = getAppVersionName();
        osVersion = Build.VERSION.RELEASE;
        manufacturer = Build.MANUFACTURER;
        model = Build.MODEL;
        height = getHeight();
        heightPixels = getHeightPixels();
        width = getWidth();
        widthPixels = getWidthPixels();
        dpi = getDPI();
        deviceName = getDeviceName();
    }

    private String getAppVersionName() {
        PackageInfo packageInfo;
        try {
            packageInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
            return packageInfo.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            Logger.d("Unable to get app version");
        }
        return null;
    }

    private int getHeightPixels() {
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        if (wm == null) {
            return 0;
        }
        DisplayMetrics dm = new DisplayMetrics();
        wm.getDefaultDisplay().getMetrics(dm);
        return dm.heightPixels;
    }

    private double getHeight() {
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        if (wm == null) {
            return 0.0;
        }
        DisplayMetrics dm = new DisplayMetrics();
        wm.getDefaultDisplay().getMetrics(dm);
        // Calculate the height in inches
        double rHeight = dm.heightPixels / dm.ydpi;
        return FlowDashUtils.getDoubleWithTwoDecimals(rHeight);
    }

    private double getWidth() {
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        if (wm == null) {
            return 0.0;
        }
        DisplayMetrics dm = new DisplayMetrics();
        wm.getDefaultDisplay().getMetrics(dm);
        // Calculate the width in inches
        double rWidth = dm.widthPixels / dm.xdpi;
        return FlowDashUtils.getDoubleWithTwoDecimals(rWidth);

    }

    private int getWidthPixels() {
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        if (wm == null) {
            return 0;
        }
        DisplayMetrics dm = new DisplayMetrics();
        wm.getDefaultDisplay().getMetrics(dm);
        return dm.widthPixels;
    }

    private int getDPI() {
        WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        if (wm == null) {
            return 0;
        }
        DisplayMetrics dm = new DisplayMetrics();
        wm.getDefaultDisplay().getMetrics(dm);
        return dm.densityDpi;
    }

    public static String getDeviceName() {
        String manufacturer = Build.MANUFACTURER;
        String model = Build.MODEL;
        if (model.startsWith(manufacturer)) {
            return capitalize(model);
        }
        return capitalize(manufacturer) + " " + model;
    }

    private static String capitalize(String str) {
        if (TextUtils.isEmpty(str)) {
            return str;
        }
        char[] arr = str.toCharArray();
        boolean capitalizeNext = true;
        String phrase = "";
        for (char c : arr) {
            if (capitalizeNext && Character.isLetter(c)) {
                phrase += Character.toUpperCase(c);
                capitalizeNext = false;
                continue;
            } else if (Character.isWhitespace(c)) {
                capitalizeNext = true;
            }
            phrase += c;
        }
        return phrase;
    }
}
