package com.flow;

import android.app.Activity;
import android.content.res.Resources;
import android.content.res.Resources.NotFoundException;
import android.graphics.Rect;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.widget.Checkable;
import android.widget.TextView;

import java.util.Arrays;


public final class ViewElement {
    private Activity activity;
    private final View view;

    public final CharSequence getContentDescription() {
        return this.view.getContentDescription();
    }

    public final Rect getBounds() {
        int[] l = new int[2];
        this.view.getLocationOnScreen(l);
        return new Rect(l[0], l[1], l[0] + this.view.getWidth(), l[1] + this.view.getHeight());
    }

    public final boolean isLongClickable() {
        return this.view.isLongClickable();
    }

    public final boolean isCheckable() {
        return this.view instanceof Checkable;
    }

    public final int getId() {
        return this.view.getId();
    }

    public final String getResourceId() {
        int id = this.getId();
        if (id != -1) {
            Resources r = this.view.getResources();
            if (id > 0 && r != null) {
                try {
                    String pkgname = null;
                    switch (id & -16777216) {
                        case 16777216:
                            pkgname = "android";
                            break;
                        case 2130706432:
                            pkgname = "app";
                            break;
                        default:
                            pkgname = r.getResourcePackageName(id);
                            if (pkgname == null)
                                throw new AssertionError("r.getResourcePackageName(id) cannot be null");
                    }

                    Object[] temp = new Object[]{pkgname, r.getResourceTypeName(id), r.getResourceEntryName(id)};
                    return String.format("%s:%s/%s", Arrays.copyOf(temp, temp.length));
                } catch (NotFoundException var8) {
                }
            }
        }

        return null;
    }

    public final String getClassName() {
        String nameCopy = this.view.getClass().getName();
        nameCopy = nameCopy.replaceAll("\\$[0-9]+", "\\$");
        int start = nameCopy.lastIndexOf('$');
        String name;
        if (start < 0) {
            name = nameCopy;
        } else {
            name = nameCopy.substring(0, start);
        }

        return name;
    }

    public final int getIndex() {
        ViewParent parentView = this.view.getParent();
        if (parentView instanceof ViewGroup) {
            int index = 0;

            for (int count = ((ViewGroup) parentView).getChildCount(); index < count; ++index) {
                if (this.view.equals(((ViewGroup) parentView).getChildAt(index))) {
                    return index;
                }
            }
        }

        return 0;
    }

    public final ViewText getText() {
        if (!(this.view instanceof TextView)) {
            return null;
        } else {
            ViewText viewText;
            CharSequence textValue;
            label35:
            {
                textValue = ((TextView) this.view).getText();
                CharSequence hintValue = ((TextView) this.view).getHint();
                CharSequence textString;
                if (textValue != null) {
                    textString = (CharSequence) textValue.toString();
                    if (textString.length() != 0) {
                        break label35;
                    }
                }

                if (hintValue != null) {
                    textString = hintValue.toString();
                    if (textString.length() != 0) {
                        return new ViewText(hintValue.toString(), true);
                    }
                }
            }
            return new ViewText(textValue.toString(), false);
        }
    }

    public final boolean isScrollable() {
        return this.view.isScrollContainer();
    }

    public final boolean isPassword() {
        return this.view instanceof TextView && this.isPasswordInputType(((TextView) this.view).getInputType());
    }

    public final String getViewTag() {
        Object tag = this.view.getTag();
        return tag != null ? tag.toString() : null;
    }


    private final boolean isPasswordInputType(int inputType) {
        int variation = inputType & 4095;
        return variation == 129 || variation == 225 || variation == 18;
    }

    public ViewElement(View view) {
        super();
        this.view = view;
    }
}
