import React, { useState, useRef } from "react";
import {
  NativeModules,
  NativeEventEmitter,
  Animated,
  TouchableWithoutFeedback,
  TouchableOpacity,
  StyleSheet,
  Dimensions,
  Easing,
} from "react-native";
import hoistStatics from "hoist-non-react-statics";
import UIManager from "react-native/Libraries/ReactNative/UIManager";
import { FAB } from "react-native-paper";
import NavigationService from "./navigationService";
import { Tooltip, Text as TooltipText } from 'react-native-elements';

let FlowDashNative = NativeModules.Flowdash;

let FlowDash;

flowdashHoC = (RootComponent) => {
  let ReactNative,
    { View, Text, findNodeHandle } = require("react-native");
  try {
    React = require("react");
  } catch (e) { }
  if (!React) {
    try {
      React = ReactNative.React;
    } catch (e) { }
    if (!React) {
      throw new Error("Unable to find the 'React' module.");
    }
  }

  return () => {
    const styles = StyleSheet.create({
      appContainer: {
        flex: 1,
      },
      container: {
        position: "absolute",
        backgroundColor: "transparent",
        top: 0,
        left: 0,
        right: 0,
        bottom: 0,
        zIndex: 2,
      },
      fab: {
        position: "absolute",
        margin: 16,
        right: 7,
        top: -50,
        zIndex: 3,
        height: 40,
        width: 40,
        justifyContent: "center",
        alignItems: "center",
      },
    });

    const FadeInView = (props) => {
      const [fadeAnim] = useState(new Animated.Value(0)); // Initial value for opacity: 0

      React.useEffect(() => {
        Animated.timing(fadeAnim, {
          toValue: 1,
          duration: 100,
          easing: Easing.quad, // TODO - Make animations configurable.
        }).start();
      }, []);

      return (
        <Animated.View // Special animatable View
          pointerEvents="box-none"
          style={{
            ...props.style,
            opacity: fadeAnim, // Bind opacity to animated value
          }}
        >
          {props.children}
        </Animated.View>
      );
    };

    const INDEPENDENT_VIEW = -5679;
    const Enhanced = class FlowdashComponent extends React.Component {
      constructor(props) {
        super(props);
        this.state = {
          inspector: false,
          focusedView: null,
          open: false,
          candidateViews: [],
          views: [],
          showInspected: false,
          currentStepIndex: 1,
          replaying: !FlowDashNative.isEditorEnabled(),
          isEditorMode: FlowDashNative.isEditorEnabled(),
          screenId: "",
        };
        this.evaluate = this.evaluate.bind(this);
      }

      // Todo : Refactor this piece.
      findViewForTouchEvent = (e) => {
        const { locationX, locationY } = e.nativeEvent.touches[0];
        UIManager.findSubviewIn(
          findNodeHandle(this._mainRef),
          [locationX, locationY],
          (nativeViewTag, left, top, width, height) => {
            this.setState({
              focusedView: {
                nativeViewTag,
                left,
                top,
                width,
                height,
                view:
                  '{"style":{"actionView":{"position":"absolute","top":"this.state.focusedView.top + this.state.focusedView.height + 15","left":"this.state.focusedView.left + this.state.focusedView.width - 90","backgroundColor":"#65f4d4","display":"flex","alignItems":"center","justifyContent":"center","width":90,"height":30,"borderRadius":15},"wrapper":{"borderColor": "#65f4d4","borderWidth": 2,"borderRadius": 5, "zIndex": 4},"actionText":{"textTransform":"uppercase","color":"black","fontWeight":"bold"},"skipActionText": {"fontSize": 16, "color": "#65f4d4","fontWeight": "bold"},"skipAction":{"position":"absolute","bottom":58,"left":15,"zIndex":2,"fontSize":16,"fontWeight":"bold"},"primaryText":{"position":"absolute","top":"this.state.focusedView.top - 30","left":"this.state.focusedView.left","color":"white","fontSize":16}},"content":{"actionText":"Next","skipAction":"Skip","primaryText":"Text describing this view."},"showIndexView":false}',
              },
              showInspected: true,
              type:1
            });
          }
        );
      };

      toggleInspector = () => {
        this.setState({
          inspector: !this.state.inspector,
          focusedView: null,
        });
      };

      shouldSetResponser = (e) => {
        this.findViewForTouchEvent(e);
        return true;
      };

      componentDidMount() {
        const eventEmitter = new NativeEventEmitter(FlowDashNative);
        this.eventListener = eventEmitter.addListener(
          "WalkthroughRequest",
          (event) => {
            if (event.loading == "true") {
              FlowDashNative.getCapturedViews(
                event.screenId,
                0,
                (nextScreen, view, type) => {
                  this.setInitView(event.screenId, view, nextScreen, type);
                }
              );
            }
          }
        );
        this.eventListener = eventEmitter.addListener(
          "TrackScreenRequest",
          (event) => {
            if (event.screenId) {
              this.setState({
                screenId: event.screenId,
                candidateViews: [],
              });
            }
          }
        );
      }

      toggleFAB = () => {
        this.setState({
          open: !this.state.open,
        });
      };

      componentWillUnmount() {
        this.eventListener.remove();
      }

      saveCurrentArea = () => {
        if (!this.state.focusedView) {
          return FlowDashNative.showToastWithText("No selection made");
        }
        let candidateViews = [
          ...this.state.candidateViews,
          this.state.focusedView,
        ];
        this.setState({
          candidateViews,
          focusedView: null,
        });
        FlowDashNative.saveView(this.state.screenId, candidateViews);
      };

      finishCapture = () => {
        FlowDashNative.publishWalkthrough();
        this.setState({
          focusedView: null,
          candidateViews: [],
        });
        this.toggleInspector();
      };

      forwardStep = () => {
        if (this.state.nextScreenId) {
          this.setState({
            currentStepIndex: 0,
            focusedView: null,
            screenId: this.state.nextScreenId,
          });
          NavigationService.navigate(
            this.state.nextScreenId,
            this.state.nextProps
          );
          return;
        }
        this.setState({
          currentStepIndex: this.state.currentStepIndex + 1,
          focusedView: null,
        });

        FlowDashNative.getCapturedViews(
          this.state.screenId,
          this.state.currentStepIndex,
          (nextScreen, view) => {
            if (Object.keys(view).length !== 0) {
              this.setState({
                focusedView: view,
                nextScreenId: nextScreen.nextScreenId,
                nextProps: nextScreen.nextProps,
              });
            } else {
              this.setState({
                focusedView: null,
                currentStepIndex: 1,
                inspector: false,
                replaying: !this.state.isEditorMode,
              });
            }
          }
        );
      };

      skipWalkthrough = () => {
        this.setState({
          focusedView: null,
          currentStepIndex: 1,
          replaying: !this.state.isEditorMode,
          inspector: false,
        });
        FlowDashNative.recordViewByUser(this.state.screenId, false);
      };

      setInitView = (screenId, view, nextScreen, type) => {
        if (Object.keys(view).length !== 0) {
          this.setState({
            type: type,
            screenId: screenId,
            nextScreenId: nextScreen.nextScreenId,
            nextProps: nextScreen.nextProps,
            views: view,
            focusedView: view,
            currentStepIndex: 1,
            inspector: true,
            replaying: true,
            showInspected: true,
            open: false,
          });
        } else {
          this.setState({
            focusedView: null,
            currentStepIndex: 1,
            inspector: false,
            replaying: !this.state.isEditorMode,
          });
        }
      };

      evaluate = (style) => {
        if (!style) return;
        for (let [key, value] of Object.entries(style)) {
          if (
            key == "top" ||
            key == "left" ||
            key == "right" ||
            key == "bottom"
          ) {
            style[key] = eval(value);
          }
        }
        return style;
      };

      viewRenderer = (view, type, direction, focusedView) => {
        let finalView;
        let tooltipStyle;
        let caretStyle;
        let defaultViewTooltipStyle = { "actionView": { "backgroundColor": "#3d72dc", "display": "flex", "alignItems": "center", "justifyContent": "center", "width": 90, "height": 30, "borderRadius": 10 }, "wrapper": { "borderColor": "#ffffff", "borderWidth": 1, "borderRadius": 5, "zIndex": 4 }, "actionText": { "color": "white" }, "skipActionText": { "fontSize": 16, "color": "#ffffff", "fontWeight": "bold" }, "skipAction": { "bottom": 58, "left": 15, "zIndex": 2, "fontSize": 16, "fontWeight": "bold", "position": "absolute" }, "primaryText": { "top": 113.61904999999999, "left": 17.142857, "color": "#747474", "fontSize": 16 }, "secondaryText": { "top": 113.61904999999999, "left": 17.142857, "color": "#747474", "fontSize": 16 }, "focussedView": { "top": 143.61905, "left": 17.142857, "width": 115.04762, "height": 115.04762 }, "indexView": { color: '#747474' } };
        let viewTooltipStyle = view.tooltipStyle ? view.tooltipStyle : defaultViewTooltipStyle;
        switch (type) {
          case "0":
            switch (direction) {
              case 0:
                tooltipStyle = {
                  left: focusedView.left - 5,
                  top: focusedView.top - 105,
                  flexDirection: 'column'
                }
                caretStyle = {
                  width: 0,
                  height: 0,
                  borderTopColor: viewTooltipStyle.tooltip ? viewTooltipStyle.tooltip.backgroundColor : '#ffffff',
                  borderTopWidth: 5,
                  borderBottomColor: 'transparent',
                  borderBottomWidth: 10,
                  borderRightColor: 'transparent',
                  borderRightWidth: 10,
                  borderLeftColor: 'transparent',
                  borderLeftWidth: 10,
                  marginLeft: 15,
                }
                break;
              case 1:
                tooltipStyle = {
                  left: focusedView.left + focusedView.width + 5,
                  top: focusedView.top + (focusedView.height / 2) - 45,
                  flexDirection: 'row-reverse'
                }
                caretStyle = {
                  width: 0,
                  height: 0,
                  borderTopColor: 'transparent',
                  borderTopWidth: 10,
                  borderBottomColor: 'transparent',
                  borderBottomWidth: 10,
                  borderRightColor: viewTooltipStyle.tooltip ? viewTooltipStyle.tooltip.backgroundColor : '#ffffff',
                  borderRightWidth: 5,
                  borderLeftColor: 'transparent',
                  borderLeftWidth: 10,
                  marginTop: 15,
                }
                break;
              case 2:
                tooltipStyle = {
                  left: focusedView.left - 5,
                  top: focusedView.top + focusedView.height + 5,
                  flexDirection: 'column-reverse'
                }
                caretStyle = {
                  width: 0,
                  height: 0,
                  borderTopColor: 'transparent',
                  borderTopWidth: 10,
                  borderBottomColor: viewTooltipStyle.tooltip ? viewTooltipStyle.tooltip.backgroundColor : '#ffffff',
                  borderBottomWidth: 5,
                  borderRightColor: 'transparent',
                  borderRightWidth: 10,
                  borderLeftColor: 'transparent',
                  borderLeftWidth: 10,
                  marginLeft: 15,
                }
                break;
              case 3:
                tooltipStyle = {
                  left: focusedView.left - 200,
                  top: focusedView.top + (focusedView.height / 2) - 45,
                  flexDirection: 'row'
                }
                caretStyle = {
                  width: 0,
                  height: 0,
                  borderTopColor: 'transparent',
                  borderTopWidth: 10,
                  borderBottomColor: 'transparent',
                  borderBottomWidth: 10,
                  borderRightColor: 'transparent',
                  borderRightWidth: 10,
                  borderLeftColor: viewTooltipStyle.tooltip ? viewTooltipStyle.tooltip.backgroundColor : '#ffffff',
                  borderLeftWidth: 5,
                  marginTop: 15,
                }
                break;
              default:
                break;
            }
            finalView = (
              <>
                <View style={{
                  position: 'absolute',
                  elevation: 5,
                  display: 'flex', justifyContent: 'center', alignItems: 'flex-start',
                  ...tooltipStyle,
                }}>
                  <View
                    style={{
                      color: 'white',
                      minWidth: 180,
                      height: 80,
                      backgroundColor: viewTooltipStyle.tooltip ? viewTooltipStyle.tooltip.backgroundColor : '#ffffff',
                      padding: 10,
                      borderRadius: 10,
                      display: 'flex',
                      flexDirection: 'column',
                      justifyContent: 'center',
                      alignItems: "center"
                    }}
                  >
                    <Text
                      style={{
                        color: viewTooltipStyle.primaryText.color,
                        fontSize: viewTooltipStyle.primaryText.fontSize,
                        fontWeight: 'bold'
                      }}
                    >
                      {view.content.primaryText}
                    </Text>
                    {view.content.secondaryText &&
                      <Text
                        style={this.evaluate(viewTooltipStyle.secondaryText)}
                      >
                        {view.content.secondaryText}
                      </Text>}
                    <View style={{
                      display: 'flex',
                      flexDirection: 'row',
                      margin: 5,
                    }}>
                      {view.showIndexView && (
                        <Text style={{
                          ...this.evaluate(viewTooltipStyle.indexView),
                          flex: 1,
                          fontWeight: 'bold',
                          position: 'relative'
                        }}>
                          Step {this.state.currentStepIndex}
                        </Text>
                      )}
                      <TouchableOpacity
                        style={{ ...this.evaluate(viewTooltipStyle.actionView), flex: 1 }}
                        onPress={this.forwardStep}
                      >
                        <Text
                          style={{ ...this.evaluate(viewTooltipStyle.actionText), textAlign: "center" }}>{view.content.actionText}</Text>
                      </TouchableOpacity>
                    </View>
                  </View>
                  <View style={caretStyle}>
                    <Text></Text>
                  </View>
                </View>
                <TouchableWithoutFeedback onPress={this.skipWalkthrough}>
                  <View style={this.evaluate(viewTooltipStyle.skipAction)}>
                    <Text style={viewTooltipStyle.skipActionText}>
                      {view.content.skipAction}
                    </Text>
                  </View>
                </TouchableWithoutFeedback>
              </>);
            break;
          case "2":
            break;
          default:
            finalView = (<>
              <Text style={this.evaluate(view.style.primaryText)}>
                {view.content.primaryText}
              </Text>
              <Text style={this.evaluate(view.style.secondaryText)}>
                {view.content.secondaryText}
              </Text>
              <TouchableWithoutFeedback onPress={this.skipWalkthrough}>
                <View style={this.evaluate(view.style.skipAction)}>
                  <Text style={view.style.skipActionText}>
                    {view.content.skipAction}
                  </Text>
                </View>
              </TouchableWithoutFeedback>
              <TouchableWithoutFeedback onPress={this.forwardStep}>
                <View style={this.evaluate(view.style.actionView)}>
                  <Text style={view.style.actionText}>
                    {view.content.actionText}
                  </Text>
                </View>
              </TouchableWithoutFeedback>
              {view.showIndexView && (
                <Text style={this.evaluate(view.style.indexView)}>
                  {this.state.currentStepIndex}
                </Text>
              )}</>);
            break;
        }
        return finalView;
      }

      render() {
        const { width, height } = Dimensions.get("window");
        const { focusedView, type, isEditorMode } = this.state;
        const isNotTiedToView = focusedView
          ? focusedView.viewIndex == INDEPENDENT_VIEW
          : null;

        const view =
          focusedView && focusedView.view ? JSON.parse(focusedView.view) : null;

        const commonStyle = {
          backgroundColor: "black",
          justifyContent: "center",
          alignItems: "center",
          opacity: 0.79,
        };
        let topViewStyle = {},
          bottomViewStyle = {},
          leftViewStyle = {},
          rightViewStyle = {},
          mainViewStyle = {};

        if (isNotTiedToView) {
          topViewStyle = {
            ...commonStyle,
            width,
            height,
          };
        } else if (focusedView) {
          topViewStyle = {
            ...commonStyle,
            width,
            height: focusedView.top - 2,
          };
          bottomViewStyle = {
            ...commonStyle,
            width,
            height: height - focusedView.height - focusedView.top - 2,
          };
          leftViewStyle = {
            ...commonStyle,
            width: focusedView.left - 2,
            height: focusedView.height + 4,
          };
          rightViewStyle = {
            ...commonStyle,
            width: width - focusedView.left - focusedView.width,
            height: focusedView.height + 4,
          };

          mainViewStyle = {
            ...commonStyle,
            width: focusedView.width + 4,
            height: focusedView.height + 4,
            backgroundColor: "transparent",
            opacity: 1,
          };
        }
        return (
          <View style={styles.appContainer} pointerEvents="box-none">
            <RootComponent
              {...this.props}
              ref={(ref) => {
                this._mainRef = ref;
              }}
            />
            {this.state.inspector && (
              <View
                onStartShouldSetResponder={
                  this.state.replaying ? null : this.shouldSetResponser
                }
                style={styles.container}
                onResponderMove={
                  this.state.replaying ? null : this.findViewForTouchEvent
                }
              />
            )}
            {!this.state.replaying && (
              <FAB.Group
                pointerEvents="box-none"
                style={{
                  position: "absolute",
                  zIndex: 999999,
                }}
                fabStyle={[styles.fab]}
                open={this.state.open}
                small={true}
                draggable
                icon={!this.state.open ? "play" : "pause"}
                actions={[
                  {
                    icon: "animation",
                    label: !this.state.inspector
                      ? "Capture"
                      : "Pause Capturing",
                    onPress: this.toggleInspector,
                  },
                  {
                    icon: "arrow-right-bold-circle",
                    label: "Save & Proceed",
                    onPress: this.saveCurrentArea,
                  },
                  {
                    icon: "flag",
                    label: "Finish",
                    onPress: this.finishCapture,
                  },
                ]}
                onStateChange={this.toggleFAB}
                onPress={this.toggleFAB}
              />
            )}
            {focusedView && this.state.showInspected && (
              // TODO - Extract this out into a HOC Component later
              // TODO - Refactor this
              <>
                <View
                  collapsable={false}
                  pointerEvents="box-none"
                  style={{
                    position: "absolute",
                    top: 0,
                    bottom: 0,
                    left: 0,
                    right: 0,
                  }}
                >
                  <FadeInView>
                    <View style={topViewStyle} pointerEvents="box-none" />
                    <View
                      style={{ flexDirection: "row" }}
                      pointerEvents="box-none"
                    >
                      <View style={leftViewStyle} pointerEvents="box-none" />
                      <View
                        style={[
                          mainViewStyle,
                          this.evaluate(view.style.wrapper),
                        ]}
                      />
                      <View style={rightViewStyle} pointerEvents="box-none" />
                    </View>
                    <View style={bottomViewStyle} pointerEvents="box-none" />
                  </FadeInView>
                </View>
              </>
            )}

            {view && this.state.showInspected && (
              <FadeInView
                style={{
                  position: "absolute",
                  top: 0,
                  bottom: 0,
                  left: 0,
                  right: 0,
                }}
              >
                {this.viewRenderer(view, type, focusedView.tooltipDirection, focusedView)}
              </FadeInView>
            )}
          </View>
        );
      }
    };
    hoistStatics(Enhanced, RootComponent);
    return <Enhanced />;
  };
};

FlowDash = flowdashHoC;

let requests = {};

let replay = (screen) => {
  if (!requests[screen]) {
    console.log("Replay attempt at screen: " + screen);
    requests[screen] = true;
    FlowDashNative.requestWalkthrough(screen, () => {
      delete requests[screen];
    });
  } else {
    console.log("Replay request in progress.");
  }
};

if (FlowDashNative) {
  Object.assign(FlowDash, {
    showToastWithText: FlowDashNative.showToastWithText,
    setNativeView: FlowDashNative.setNativeView,
    replay: replay,
    trackCurrentScreen: FlowDashNative.trackCurrentScreen,
    setTopLevelNavigator: NavigationService.setTopLevelNavigator,
  });
}

export default FlowDash;